#
# Copyright 2019 AXA Group Operations S.A.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import logging
import pandas as pd
from io import StringIO

class ParsrOutputInterpreter(object):
	def __init__(self, object=None):
		logging.basicConfig(level=logging.DEBUG, format='%(name)s - %(levelname)s - %(message)s')
		self.object = None
		if object is not None:
			self.load_object(object)
	
	def __get_text_types(self):
		return ['word', 'line', 'character', 'paragraph', 'heading']

	def __text_objects_none_page(self, txts, page_number_none):
		for page in self.object['pages']:
			for element in page['elements']:
				if element['type'] in self.__get_text_types():
					txts.append(element)
	
	def __get_text_objects(self, page_number=None):
		texts = []
		if page_number is not None:
			page = self.get_page(page_number)
			if page is None:
				logging.error("Cannot get text elements for the requested page; Page {} not found".format(page_number))
				return None
			else:
				for element in page['elements']:
					if element['type'] in self.__get_text_types():
						texts.append(element)
		else:
			texts = self.__text_object_none_page(texts, page_number)
			
		return texts
	
	def __text_from_text_object(self, text_object:dict) -> str:
		result = ""
		if (text_object['type'] in ['paragraph', 'heading']) or (text_object['type'] in ['line']):
			for i in text_object['content']:
				result += self.__text_from_text_object(i)
		elif text_object['type'] in ['word']:
			if type(text_object['content']) is list:
				for i in text_object['content']:
					result += self.__text_from_text_object(i)
			else:
				result += text_object['content']
				result += ' '
		elif text_object['type'] in ['character']:
			result += text_object['content']
		return result
	
	def load_object(self, object):
		self.object = object

	def get_page(self, page_number):
		for p in self.object['pages']:
			if p['pageNumber'] == page_number:
				return p
		logging.error("Page {} not found".format(page_number))
		return None

	def get_text(self, page_number:int=None) -> str:
		final_text = ""
		for text_obj in self.__get_text_objects(page_number):
			final_text += self.__text_from_text_object(text_obj)
			final_text += "\n\n"
		return final_text



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['parsr_client']

package_data = \
{'': ['*']}

install_requires = \
['diff_match_patch>=20181111,<20181112',
 'pandas>=1.0.3,<2.0.0',
 'requests>=2.23.0,<3.0.0',
 'semver>=2.9.1,<3.0.0',
 'sxsdiff>=0.3.0,<0.4.0']

setup_kwargs = {
    'name': 'parsr-client',
    'version': '3.2.0',
    'description': 'Python client for Parsr - Transforms PDF, Documents and Images into Enriched Structured Data',
    'long_description': '# Parsr Client\n\nProvides a python interface to the Parsr toolvia its API, which transforms PDF, documents and images into enriched, structured data.\n    \n',
    'author': 'AXA Group Operations S.A.',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'http://par.sr',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



#
# Copyright 2020 AXA Group Operations S.A.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from glob import glob
from itertools import chain
import os
import sys
import json
import time
import ast
from enum import Enum

import semver

import pandas as pd
import requests
from io import StringIO

import diff_match_patch
from sxsdiff import DiffCalculator
from sxsdiff.generators.github import GitHubStyledGenerator


class ParsrClient():
	def __init__(self, server, revision_history:dict={}):
		self.revision_history = revision_history
		self.set_server(server)
		self.set_current_request_id("")

	def __supported_input_files(self) -> list:
		return ['*.pdf', '*.jpg', '*.jpeg', '*.png', '*.tiff', '*.tif',]

	def set_server(self, server:str):
		self.server = server

	def set_current_request_id(self, request_id:str):
		self.request_id = request_id

	def send_document(self, file:str, config:str, server:str="", document_name:str=None, revision:str='major', wait_till_finished:bool=False, refresh_period=2, save_request_id:bool=False, silent:bool=True) -> dict:
		if server == "":
			if self.server == "":
				raise Exception('No server address provided')
			else:
				server = self.server
		packet = {
			'file': (file, open(file, 'rb'), 'application/pdf'),
			'config': (config, open(config, 'rb'), 'application/json'),
		}
		r = requests.post('http://'+server+'/api/v1/document', files=packet)
		jobId = r.text
		if not document_name:
			document_name = os.path.splitext(os.path.basename(file))[0]
		if document_name not in self.revision_history:
			self.revision_history[document_name] = { str(semver.VersionInfo.parse('1.0.0')): jobId }
		else:
			latest_revision = max(semver.VersionInfo.parse(i) for i in list(self.revision_history[document_name].keys()))
			if revision == 'major':
				new_revision = latest_revision.bump_major()
			elif revision == 'minor':
				new_revision = latest_revision.bump_minor()
			self.revision_history[document_name][str(new_revision)] = jobId
		if save_request_id:
			self.set_current_request_id(jobId)
		if not wait_till_finished:
			return {'file': file, 'config': config, 'status_code': r.status_code, 'server_response': r.text}
		else:
			print('> Polling server for the job {}...'.format(jobId))
			server_status_response = self.get_status(jobId)['server_response']
			while ('progress-percentage' in server_status_response):
				if not silent:
					print('>> Progress percentage: {}'.format(server_status_response['progress-percentage']))
				time.sleep(refresh_period)
				server_status_response = self.get_status(jobId)['server_response']
			print('>> Job done!')
			return {'file': file, 'config': config, 'status_code': r.status_code, 'server_response': r.text}

	def get_request_id(self, document_name:str, revision:str) -> str:
		if document_name in self.revision_history:
			if revision in self.revision_history[document_name].keys():
				return self.revision_history[document_name][revision]
			else:
				print('Revision {} not found for document {}'.format(revision, document_name))
		else:
			print('Document name {} not found'.format(document_name))
		return ""

	def get_revisions(self, document_name:str) -> list:
		if document_name in self.revision_history:
			return list(self.revision_history[document_name].keys())
		else:
			return []

	def get_document_name_from_request_id(self, request_id:str) -> str:
		for document_name in list(self.revision_history.keys()):
			if request_id in [self.revision_history[document_name][i] for i in list(self.revision_history[document_name])]:
				return document_name
		return ""

	def compare_revisions(self, document_name:str, revisions:list = [], pretty_html:bool = False) -> list:
		diffs = []
		if len(revisions) == 0:
			revisions = self.get_revisions(document_name)
		request_ids = [self.get_request_id(document_name, i) for i in revisions]
		for i in range(0, len(request_ids) - 1):
			request_id1 = request_ids[i]
			request_id2 = request_ids[i + 1]
			md1 = self.get_markdown(request_id1)
			md2 = self.get_markdown(request_id2)

			if pretty_html:
				sxsdiff_result = DiffCalculator().run(md1, md2)
				html_store = StringIO()
				GitHubStyledGenerator(file=html_store).run(sxsdiff_result)
				html_diff = html_store.getvalue()
				diffs.append(html_diff)
			else:
				dmp = diff_match_patch.diff_match_patch()
				diff = dmp.diff_main(md1, md2)
				dmp.diff_cleanupSemantic(diff)
				diffs.append(diff)
		return diffs

	def send_documents_folder(self, folder:str, config:str, server:str="") -> list:
		if server == "":
			if self.server == "":
				raise Exception('No server address provided')
			else:
				server = self.server
		responses = []
		os.chdir(folder)
		files = [glob.glob(e) for e in self.__supported_input_files()]
		files_flat = list(chain.from_iterable(files))
		for file in files_flat:
			packet = {
				'file': (file, open(file, 'rb'), 'application/pdf'),
				'config': (config, open(config, 'rb'), 'application/json'),
			}
			r = requests.post('http://'+server+'/api/v1/document', files=packet)
			responses.append({'file': file, 'config': config, 'status_code': r.status_code, 'server_response': r.text})
		return responses

	def get_status(self, request_id:str="", server:str=""):
		if server == "":
			if self.server == "":
				raise Exception('No server address provided')
			else:
				server = self.server
		if request_id == "":
			if self.request_id == "":
				raise Exception('No request ID provided')
			else:
				request_id = self.request_id
		if self.server == "":
			raise Exception('No server address provided')
		r = requests.get('http://{}/api/v1/queue/{}'.format(server, request_id))
		return {'request_id': request_id, 'server_response': json.loads(r.text)}

	def get_json(self, request_id:str="", server:str=""):
		if server == "":
			if self.server == "":
				raise Exception('No server address provided')
			else:
				server = self.server
		if request_id == "":
			if self.request_id == "":
				raise Exception('No request ID provided')
			else:
				request_id = self.request_id
		r = requests.get('http://{}/api/v1/json/{}'.format(server, request_id))
		if r.text != "":
			return r.json()
		else:
			return {'request_id': request_id, 'server_response': r.json()}

	def get_markdown(self, request_id:str="", server:str=""):
		if server == "":
			if self.server == "":
				raise Exception('No server address provided')
			else:
				server = self.server
		if request_id == "":
			if self.request_id == "":
				raise Exception('No request ID provided')
			else:
				request_id = self.request_id
		r = requests.get('http://{}/api/v1/markdown/{}'.format(server, request_id))
		if r.text != "":
			return r.text
		else:
			return {'request_id': request_id, 'server_response': r.text}

	def get_text(self, request_id:str="", server:str=""):
		if server == "":
			if self.server == "":
				raise Exception('No server address provided')
			else:
				server = self.server
		if request_id == "":
			if self.request_id == "":
				raise Exception('No request ID provided')
			else:
				request_id = self.request_id
		r = requests.get('http://{}/api/v1/text/{}'.format(server, request_id))
		if r.text != "":
			return r.text
		else:
			return {'request_id': request_id, 'server_response': r.text}

	def get_tables_info(self, request_id:str=""):
		return [(table.rsplit('/')[-2], table.rsplit('/')[-1]) for table in ast.literal_eval(self.get_table(request_id=request_id).columns[0])]

	def get_table(self, request_id:str="", page=None, table=None, seperator=";", server:str="", column_names:list=None):
		if server == "":
			if self.server == "":
				raise Exception('No server address provided')
			else:
				server = self.server
		if request_id == "":
			if self.request_id == "":
				raise Exception('No request ID provided')
			else:
				request_id = self.request_id
		if page is None and table is None:
			r = requests.get('http://{}/api/v1/csv/{}'.format(server, request_id))
		else:
			r = requests.get('http://{}/api/v1/csv/{}/{}/{}'.format(server, request_id, page, table))
		if r.text != "":
			try:
				df = pd.read_csv(StringIO(r.text), sep=seperator, names=column_names)
				df.loc[:, ~df.columns.str.match('Unnamed')]
				df = df.where((pd.notnull(df)), " ")
				return df
			except Exception as e:
				return r.text
		else:
			return r.text
